@@ -1,31 +1,37 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Generic
-from typing import Iterable
-from typing import Iterator
-from typing import Mapping
-from typing import Protocol
-from typing import Sequence
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Generic,
+    Iterable,
+    Iterator,
+    Mapping,
+    Protocol,
+    Sequence,
+)
 
-from narwhals._compliant.any_namespace import CatNamespace
-from narwhals._compliant.any_namespace import DateTimeNamespace
-from narwhals._compliant.any_namespace import ListNamespace
-from narwhals._compliant.any_namespace import StringNamespace
-from narwhals._compliant.any_namespace import StructNamespace
-from narwhals._compliant.typing import CompliantSeriesT_co
-from narwhals._compliant.typing import EagerSeriesT_co
-from narwhals._compliant.typing import NativeSeriesT
-from narwhals._compliant.typing import NativeSeriesT_co
-from narwhals._translate import FromIterable
-from narwhals._translate import FromNative
-from narwhals._translate import NumpyConvertible
-from narwhals.utils import _StoresCompliant
-from narwhals.utils import _StoresNative
-from narwhals.utils import is_compliant_series
-from narwhals.utils import is_sized_multi_index_selector
-from narwhals.utils import unstable
+from narwhals._compliant.any_namespace import (
+    CatNamespace,
+    DateTimeNamespace,
+    ListNamespace,
+    StringNamespace,
+    StructNamespace,
+)
+from narwhals._compliant.typing import (
+    CompliantSeriesT_co,
+    EagerSeriesT_co,
+    NativeSeriesT,
+    NativeSeriesT_co,
+)
+from narwhals._translate import FromIterable, FromNative, NumpyConvertible, ToNarwhals
+from narwhals.utils import (
+    _StoresCompliant,
+    _StoresNative,
+    is_compliant_series,
+    is_sized_multi_index_selector,
+    unstable,
+)
 
 if TYPE_CHECKING:
     from types import ModuleType
@@ -36,26 +42,25 @@ if TYPE_CHECKING:
     from typing_extensions import Self
 
     from narwhals._compliant.dataframe import CompliantDataFrame
-    from narwhals._compliant.expr import CompliantExpr
-    from narwhals._compliant.expr import EagerExpr
-    from narwhals._compliant.namespace import CompliantNamespace
-    from narwhals._compliant.namespace import EagerNamespace
+    from narwhals._compliant.expr import CompliantExpr, EagerExpr
+    from narwhals._compliant.namespace import CompliantNamespace, EagerNamespace
     from narwhals.dtypes import DType
-    from narwhals.typing import ClosedInterval
-    from narwhals.typing import FillNullStrategy
-    from narwhals.typing import Into1DArray
-    from narwhals.typing import MultiIndexSelector
-    from narwhals.typing import NonNestedLiteral
-    from narwhals.typing import NumericLiteral
-    from narwhals.typing import RankMethod
-    from narwhals.typing import RollingInterpolationMethod
-    from narwhals.typing import SizedMultiIndexSelector
-    from narwhals.typing import TemporalLiteral
-    from narwhals.typing import _1DArray
-    from narwhals.typing import _SliceIndex
-    from narwhals.utils import Implementation
-    from narwhals.utils import Version
-    from narwhals.utils import _FullContext
+    from narwhals.series import Series
+    from narwhals.typing import (
+        ClosedInterval,
+        FillNullStrategy,
+        Into1DArray,
+        MultiIndexSelector,
+        NonNestedLiteral,
+        NumericLiteral,
+        RankMethod,
+        RollingInterpolationMethod,
+        SizedMultiIndexSelector,
+        TemporalLiteral,
+        _1DArray,
+        _SliceIndex,
+    )
+    from narwhals.utils import Implementation, Version, _FullContext
 
 __all__ = ["CompliantSeries", "EagerSeries"]
 
@@ -64,6 +69,7 @@ class CompliantSeries(
     NumpyConvertible["_1DArray", "Into1DArray"],
     FromIterable,
     FromNative[NativeSeriesT],
+    ToNarwhals["Series[NativeSeriesT]"],
     Protocol[NativeSeriesT],
 ):
     _implementation: Implementation
@@ -105,6 +111,8 @@ class CompliantSeries(
         name: str = "",
         dtype: DType | type[DType] | None = None,
     ) -> Self: ...
+    def to_narwhals(self) -> Series[NativeSeriesT]:
+        return self._version.series(self, level="full")
 
     # Operators
     def __add__(self, other: Any) -> Self: ...
@@ -180,7 +188,7 @@ class CompliantSeries(
         *,
         bin_count: int | None,
         include_breakpoint: bool,
-    ) -> CompliantDataFrame[Self, Any, Any]: ...
+    ) -> CompliantDataFrame[Self, Any, Any, Any]: ...
     def head(self, n: int) -> Self: ...
     def is_between(
         self, lower_bound: Any, upper_bound: Any, closed: ClosedInterval
@@ -195,6 +203,7 @@ class CompliantSeries(
     def is_unique(self) -> Self: ...
     def item(self, index: int | None) -> Any: ...
     def len(self) -> int: ...
+    def log(self, base: float) -> Self: ...
     def max(self) -> Any: ...
     def mean(self) -> float: ...
     def median(self) -> float: ...
@@ -214,34 +223,16 @@ class CompliantSeries(
         return_dtype: DType | type[DType] | None,
     ) -> Self: ...
     def rolling_mean(
-        self,
-        window_size: int,
-        *,
-        min_samples: int,
-        center: bool,
+        self, window_size: int, *, min_samples: int, center: bool
     ) -> Self: ...
     def rolling_std(
-        self,
-        window_size: int,
-        *,
-        min_samples: int,
-        center: bool,
-        ddof: int,
+        self, window_size: int, *, min_samples: int, center: bool, ddof: int
     ) -> Self: ...
     def rolling_sum(
-        self,
-        window_size: int,
-        *,
-        min_samples: int,
-        center: bool,
+        self, window_size: int, *, min_samples: int, center: bool
     ) -> Self: ...
     def rolling_var(
-        self,
-        window_size: int,
-        *,
-        min_samples: int,
-        center: bool,
-        ddof: int,
+        self, window_size: int, *, min_samples: int, center: bool, ddof: int
     ) -> Self: ...
     def round(self, decimals: int) -> Self: ...
     def sample(
@@ -262,20 +253,15 @@ class CompliantSeries(
     def to_arrow(self) -> pa.Array[Any]: ...
     def to_dummies(
         self, *, separator: str, drop_first: bool
-    ) -> CompliantDataFrame[Self, Any, Any]: ...
-    def to_frame(self) -> CompliantDataFrame[Self, Any, Any]: ...
+    ) -> CompliantDataFrame[Self, Any, Any, Any]: ...
+    def to_frame(self) -> CompliantDataFrame[Self, Any, Any, Any]: ...
     def to_list(self) -> list[Any]: ...
     def to_pandas(self) -> pd.Series[Any]: ...
     def to_polars(self) -> pl.Series: ...
     def unique(self, *, maintain_order: bool) -> Self: ...
     def value_counts(
-        self,
-        *,
-        sort: bool,
-        parallel: bool,
-        name: str | None,
-        normalize: bool,
-    ) -> CompliantDataFrame[Self, Any, Any]: ...
+        self, *, sort: bool, parallel: bool, name: str | None, normalize: bool
+    ) -> CompliantDataFrame[Self, Any, Any, Any]: ...
     def var(self, *, ddof: int) -> float: ...
     def zip_with(self, mask: Any, other: Any) -> Self: ...
 
