@@ -2,15 +2,14 @@ from __future__ import annotations
 
 from typing import TYPE_CHECKING
 
-from narwhals._duckdb.utils import native_to_narwhals_dtype
+from narwhals._duckdb.utils import DeferredTimeZone, native_to_narwhals_dtype
 from narwhals.dependencies import get_duckdb
 
 if TYPE_CHECKING:
     from types import ModuleType
 
     import duckdb
-    from typing_extensions import Never
-    from typing_extensions import Self
+    from typing_extensions import Never, Self
 
     from narwhals.dtypes import DType
     from narwhals.utils import Version
@@ -29,7 +28,11 @@ class DuckDBInterchangeSeries:
 
     @property
     def dtype(self) -> DType:
-        return native_to_narwhals_dtype(self._native_series.types[0], self._version)
+        return native_to_narwhals_dtype(
+            self._native_series.types[0],
+            self._version,
+            DeferredTimeZone(self._native_series),
+        )
 
     def __getattr__(self, attr: str) -> Never:
         msg = (  # pragma: no cover
